﻿using CallCenter.Caches;
using CallCenter.Calls;
using CallCenter.Realtimes;
using CallCenter.Share.Enums;
using CallCenter.Share.Notifications;
using MediatR;
using Microsoft.Extensions.Logging;
using System.Text.Json;

namespace CallCenter.Application.Handlers
{
    public class AnsweredVisitorToExtNotificationHandler:INotificationHandler<AnsweredVisitorToExtNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly IRealtimeService _realtimeService;
        private readonly IUserCacheManager _userCacheManager;
        private readonly ICallCacheManager _callCacheManager;
        private readonly ILogger<AnsweredVisitorToExtNotificationHandler> _logger;

        public AnsweredVisitorToExtNotificationHandler(ICallRepository callRepository, ICallDetailRepository callDetailRepository,IRealtimeService realtimeService,IUserCacheManager userCacheManager, ICallCacheManager callCacheManager,ILogger<AnsweredVisitorToExtNotificationHandler> logger)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
            _realtimeService = realtimeService;
            _userCacheManager = userCacheManager;
            _callCacheManager = callCacheManager;
            _logger = logger;
        }

        public async Task Handle(AnsweredVisitorToExtNotification notification, CancellationToken cancellationToken)
        {
            var model = await _callRepository.GetAsync(
                x => x.ConversationId == notification.Visitor.Id &&
                     x.FromNo == notification.Visitor.From && x.CreationTime >= DateTime.Now.AddHours(-2), cancellationToken);
            if (model != null)
            {
                model.CallStatus = ECallStatus.Answered;
                //model.ToNo = notification.TelNo;
                model.ConnSeaTime = DateTime.Now;
                await _callRepository.UpdateAsync(model, cancellationToken);
                var detail = new CallDetail()
                {
                    CallId = model.Id,
                    CallStatus = ECallStatus.Answered,
                    OMCallId = notification.Visitor.CallId,
                    ConversationId = notification.Visitor.Id,
                    EventName = notification.Attribute,
                    AnswerNo = notification.TelNo,
                    FromNo = notification.Visitor.From,
                    ToNo = notification.Visitor.To,
                };
                await _callDetailRepository.AddAsync(detail, cancellationToken);


                //处理排队缓存数据 TODO
                _callCacheManager.RemoveCallCache(model.Id);
                _logger.LogInformation(JsonSerializer.Serialize(_callCacheManager.GetCallQueueList()));
                await _realtimeService.CallQueueAsync(_callCacheManager.GetCallQueueList(), cancellationToken);

                var callDetail = _callDetailRepository.GetAsync(x => x.CallId == model.Id && x.EventName == "ANSWER");
                if (callDetail == null)
                {
                    //调用业务通知 通知前端
                    var workModel = _userCacheManager.GetWorkByTel(notification.TelNo);
                    if (workModel != null)
                    {
                        await _realtimeService.AnsweredAsync(workModel.UserId, new AnseredDto() { ConversationId = notification.Visitor.Id, From = notification.Visitor.From, Id = model.Id, To = notification.TelNo, CallType = ECallType.VisitorCallIn }, cancellationToken);
                    }
                }
            }
        }
    }
}
